package com.yonge.cooleshow.admin.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetApplicationExtendClientWrapper;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.yonge.cooleshow.biz.dal.dto.search.MusicSheetSearch;
import com.yonge.cooleshow.biz.dal.dto.search.StudentMusicSheetOrderSearch;
import com.yonge.cooleshow.biz.dal.dto.search.TeacherMusicSheetSearch;
import com.yonge.cooleshow.biz.dal.enums.AuthStatusEnum;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.OrderTypeEnum;
import com.yonge.cooleshow.biz.dal.service.MusicSheetService;
import com.yonge.cooleshow.biz.dal.service.SysUserService;
import com.yonge.cooleshow.biz.dal.service.cbs.CbsMusicScoreService;
import com.yonge.cooleshow.biz.dal.vo.*;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

@RestController
@RequestMapping("${app-config.url.admin:}/")
@Api(tags = "曲谱表 API接口")
public class MusicSheetController extends BaseController {
    @Resource
    private SysUserService sysUserService;
	@Resource
	private MusicSheetService musicSheetService;
	@Resource
	private CbsMusicScoreService cbsMusicScoreService;

    @GetMapping("music/sheet/detail/{id}")
    @ApiOperation(value = "详情", notes = "传入id")
    @PreAuthorize("@pcs.hasPermissions('music/sheet/detail')")
    public HttpResponseResult<MusicSheetDetailVo> detail(@ApiParam(value = "曲谱编号", required = true) @PathVariable("id") Long id) {
        return succeed(musicSheetService.detail(id, sysUserService.getUser(), ClientEnum.SYSTEM));
    }

    @ApiOperation(value = "分页查询")
    @PostMapping(value="music/sheet/list")
    @PreAuthorize("@pcs.hasPermissions('music/sheet/list')")
    public R<com.microsvc.toolkit.common.response.paging.PageInfo<CbsMusicSheetWrapper.MusicSheetApplication>> list(@RequestBody CbsMusicSheetWrapper.MusicSheetApplicationQuery query) {
        return cbsMusicScoreService.selectPage(query);
    }

    @ApiOperation(value = "分页查询", httpMethod="POST", consumes="application/json", produces="application/json")
    @PostMapping(value="/list", consumes="application/json", produces="application/json")
    @PreAuthorize("@pcs.hasPermissions('music/sheet/list')")
    public HttpResponseResult<PageInfo<MusicSheetVo>> list(@RequestBody MusicSheetSearch query) {
        query.setAuditStatus(AuthStatusEnum.PASS);
        IPage<MusicSheetVo> musicSheetVoIPage = musicSheetService.selectCbsPage(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(musicSheetVoIPage));
    }

    @ApiOperation(value = "修改应用扩展信息")
    @PreAuthorize("@pcs.hasPermissions('music/sheet/update', {'BACKEND'})")
    @PostMapping("music/sheet/update")
    public R<JSONObject> update(@RequestBody CbsMusicSheetApplicationExtendClientWrapper.AddMusicSheetApplicationExtend addMusicSheetApplicationExtend) {
        cbsMusicScoreService.update(addMusicSheetApplicationExtend);
        return R.defaultR();
    }

    @ApiOperation(value = "启/停用")
    @PreAuthorize("@pcs.hasPermissions('music/sheet/status', {'BACKEND'})")
    @PostMapping("music/sheet/status")
    public R<JSONObject> status(@RequestBody CbsMusicSheetApplicationExtendClientWrapper.MusicSheetApplicationExtendStatus status) {
        cbsMusicScoreService.updateStatus(status);
        return R.defaultR();
    }

    @ApiOperation(value = "详情", notes = "曲目表-根据详情ID查询单条, 传入id")
    @PreAuthorize("@pcs.hasPermissions('musicSheet/cbsDetail', {'BACKEND'})")
    @GetMapping("musicSheet/cbsDetail/{id}")
    public R<CbsMusicSheetWrapper.MusicSheet> cbsDetail(@PathVariable("id") Long id) {
        return cbsMusicScoreService.musicSheetDetail(id);
    }

    @PostMapping(value = "music/sheet/student")
    @ApiOperation(value = "学生详情-乐谱")
    @PreAuthorize("@pcs.hasPermissions('music/sheet/student')")
    public HttpResponseResult<PageInfo<StudentMusicSheetOrderVo>> student(@Valid @RequestBody StudentMusicSheetOrderSearch query) {
        return succeed(PageUtil.pageInfo(musicSheetService.selectStudentOrderPage(PageUtil.getPage(query),query)));
    }

    @PostMapping(value = "music/sheet/teacher")
    @ApiOperation(value = "老师详情-乐谱")
    @PreAuthorize("@pcs.hasPermissions('music/sheet/teacher')")
    public HttpResponseResult<PageInfo<TeacherMusicSheetVo>> teacher(@Valid @RequestBody TeacherMusicSheetSearch query) {
        return succeed(PageUtil.pageInfo(musicSheetService.selectTeacherPage(PageUtil.getPage(query),query)));
    }

    @PostMapping(value = "music/sheet/buySendMessage")
    @ApiOperation(value = "发送学生购买老师曲目信息")
    public HttpResponseResult<Boolean> buySendMessage(@Valid @RequestBody MusicSheetStudentByMessage param) {
        return status(musicSheetService.sendBuyMessage(param.getTeacherId(), param.getStudentId(), param.getMusicSheetId(),
                                                       ClientEnum.STUDENT, OrderTypeEnum.MUSIC));
    }
}
