package com.yonge.cooleshow.biz.dal.service.cbs.impl;

import com.alibaba.fastjson.JSONObject;
import com.dayaedu.cbs.openfeign.client.MusicFeignClientService;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetApplicationExtendClientWrapper;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetWrapper;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.microsvc.toolkit.common.response.template.R;
import com.yonge.cooleshow.biz.dal.entity.MusicSheet;
import com.yonge.cooleshow.biz.dal.enums.AuthStatusEnum;
import com.yonge.cooleshow.biz.dal.service.MusicSheetService;
import com.yonge.cooleshow.biz.dal.service.cbs.CbsMusicScoreService;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.exception.BizException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@Service
public class CbsMusicScoreServiceImpl implements CbsMusicScoreService {

    @Value("${openfeign-client.app-id:1745637981387108354}")
    public Long applicationId;
    @Resource
    private MusicFeignClientService musicFeignClientService;
    @Resource
    private MusicSheetService musicSheetService;

    @Override
    public R<PageInfo<CbsMusicSheetWrapper.MusicSheetApplication>> selectPage(CbsMusicSheetWrapper.MusicSheetApplicationQuery query) {
        query.setApplicationId(applicationId);
        return musicFeignClientService.musicSheetPageByApplication(query);
    }

    @Override
    public Long musicSheetAdd(CbsMusicSheetWrapper.AddMusicSheet addMusicSheet) {
        List<CbsMusicSheetApplicationExtendClientWrapper.AddMusicSheetApplicationExtend> addMusicSheetApplicationExtend = addMusicSheet.getAddMusicSheetApplicationExtend();
        if(CollectionUtils.isEmpty(addMusicSheetApplicationExtend)){
            throw new BizException("曲目应用不能为空");
        }
        if(addMusicSheet.getId() == null){
            addMusicSheetApplicationExtend.forEach(item->{
                item.setApplicationId(applicationId);
                item.setStatus(false);
            });
            addMusicSheet.setDelFlag(true);
            R<Long> longR = musicFeignClientService.musicSheetAdd(addMusicSheet);
            if (longR.getCode() != 200) {
                throw new BizException("添加失败,{}",longR.getMessage());
            }
            Long cbsMusicSheetId = longR.getData();
            //获取本地曲目修改审核状态
            MusicSheet musicSheet = musicSheetService.lambdaQuery().eq(MusicSheet::getCbsMusicSheetId, cbsMusicSheetId).one();
            if(musicSheet == null){
                throw new BizException("曲目上传失败，请联系管理员");
            }
            musicSheet.setAuditStatus(AuthStatusEnum.DOING);
            musicSheetService.updateById(musicSheet);
        }else {
            MusicSheet musicSheet = musicSheetService.lambdaQuery().eq(MusicSheet::getId, addMusicSheet.getId()).one();
            if(musicSheet.getState() == YesOrNoEnum.YES){
                throw new BizException("曲目已上架，不能修改");
            }
            musicSheet.setAuditStatus(AuthStatusEnum.DOING);
            musicSheetService.updateById(musicSheet);
        }
        return null;
    }

    @Override
    public void update(CbsMusicSheetApplicationExtendClientWrapper.AddMusicSheetApplicationExtend addMusicSheetApplicationExtend) {
        addMusicSheetApplicationExtend.setApplicationId(applicationId);
        R<JSONObject> jsonObjectR = musicFeignClientService.musicSheetApplicationExtendUpdate(addMusicSheetApplicationExtend);
        if (jsonObjectR.getCode() != 200) {
            throw new BizException("修改失败,{}",jsonObjectR.getMessage());
        }
    }

    @Override
    public void updateStatus(CbsMusicSheetApplicationExtendClientWrapper.MusicSheetApplicationExtendStatus status) {
        R<JSONObject> jsonObjectR = musicFeignClientService.musicSheetApplicationExtendStatus(status);
        if (jsonObjectR.getCode() != 200) {
            throw new BizException("修改状态失败,{}",jsonObjectR.getMessage());
        }
    }

    @Override
    public R<CbsMusicSheetWrapper.MusicSheet> musicSheetDetail(Long id) {
        return musicFeignClientService.musicSheetDetail(id, applicationId);
    }
}
